<?php

namespace App\domain\amadeus;

use App\domain\amadeus\traits\AmadeusParamsTraits;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Session;

abstract class AmadeusRequestStateless
{
    use AmadeusParamsTraits;

    protected mixed $mid;
    protected mixed $nonce;
    protected mixed $timestamp;
    protected mixed $passSHA;

    public function __construct()
    {
        $amadeusHeaders = new AmadeusHeaders();
        ['nonce' => $this->nonce, 'utcTimestamp' => $this->timestamp, 'mid' => $this->mid, 'passSHA' => $this->passSHA] = $amadeusHeaders->getHeaders();
    }

    protected function createCurlRequest(): string|bool
    {
        $curl = curl_init();

        $caBundlePath = base_path('cacert.pem');

        curl_setopt_array($curl, array(
            CURLOPT_URL => $this->getToAddress(),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => $this->generateXMLRequest(),
            CURLOPT_HTTPHEADER => array(
                'Content-Type: text/xml; charset=utf-8',
                'SOAPAction: ' . $this->action()
            ),
            CURLOPT_CAINFO => $caBundlePath,
        ));

        $response = curl_exec($curl);

        if (curl_errno($curl)) {
            Log::info('Curl error: ' . curl_error($curl));
        }

        curl_close($curl);

        $this->createSession($response);

        return $response;
    }

    protected function createSession($response): void
    {
        $xmlObject = simplexml_load_string($response);

        // Register the 'awsse' namespace
        $xmlObject->registerXPathNamespace('awsse', 'http://xml.amadeus.com/2010/06/Session_v3');

        $sessionId = (string)$xmlObject->xpath('//awsse:Session/awsse:SessionId')[0];
        $sequenceNumber = (string)$xmlObject->xpath('//awsse:Session/awsse:SequenceNumber')[0];
        $securityToken = (string)$xmlObject->xpath('//awsse:Session/awsse:SecurityToken')[0];

        $expirationTime = 24 * 60;

        Session::put('amadeus-session-id', $sessionId, $expirationTime);
        Session::put('amadeus-sequence-number', $sequenceNumber, $expirationTime);
        Session::put('amadeus-security-token', $securityToken, $expirationTime);
    }

    protected function xmlResponseBodyToArray($response): array
    {
        $xmlObject = simplexml_load_string($response);

        $body = $xmlObject->xpath('soap:Body');

        return json_decode(json_encode($body), true);
    }

    protected function xmlHeader(): string
    {
        return '
            <soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:ses="http://xml.amadeus.com/2010/06/Session_v3">
                <soap:Header>
                  <ses:Session TransactionStatusCode="Start" />
                    <add:MessageID xmlns:add="http://www.w3.org/2005/08/addressing">' . $this->mid . '</add:MessageID>
                    <add:Action xmlns:add="http://www.w3.org/2005/08/addressing">' . $this->action() . '</add:Action>
                    <add:To xmlns:add="http://www.w3.org/2005/08/addressing">' . $this->getToAddress() . '</add:To>
                    <link:TransactionFlowLink xmlns:link="http://wsdl.amadeus.com/2010/06/ws/Link_v1" />
                    <oas:Security xmlns:oas="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
                      <oas:UsernameToken xmlns:oas1="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" oas1:Id="UsernameToken-1">
                        <oas:Username>WSCRLCAR</oas:Username>
                        <oas:Nonce EncodingType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"> ' . $this->nonce . '</oas:Nonce>
                        <oas:Password Type="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordDigest">' . $this->passSHA . '</oas:Password>
                        <oas1:Created>' . $this->timestamp . '</oas1:Created>
                      </oas:UsernameToken>
                    </oas:Security>
                    <AMA_SecurityHostedUser xmlns="http://xml.amadeus.com/2010/06/Security_v1">
                      <UserID AgentDutyCode="SU" POS_Type="1" PseudoCityCode="LONU128OS" RequestorType="U" />
                    </AMA_SecurityHostedUser>
                </soap:Header>
        ';
    }

    abstract protected function generateXMLRequest(): string;

    abstract protected function action(): string;
}
