<?php

namespace App\domain\amadeus;

use App\domain\amadeus\traits\AmadeusParamsTraits;
use App\Models\AmaCombinedOffers;
use Illuminate\Support\Collection;

class QueryCombinedOffersPageAction
{
    use AmadeusParamsTraits;
    protected string $catalogId;

    public function execute(string $requestId, int $page): array
    {
        $this->catalogId = $requestId;

        $combinedOffers = $this->getCombinedOffersOfPage($requestId, $page);

        return $combinedOffers->toArray();
    }

    protected function getCombinedOffersOfPage(string $requestId, int $page): Collection
    {
        $records = AmaCombinedOffers::query()
            ->where('requestId', $requestId)
            ->orderBy('bestCombinablePrice')
            ->skip(($page - 1) * $this->getOffersPerPage())
            ->take($this->getOffersPerPage())
            ->get();

        $records = $records
            ->pluck('offers')
            ->flatMap(function ($offers) {
                return json_decode($offers);
            });

        return $records->groupBy('combinabilityCode');
    }
}
