<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Illuminate\Support\Facades\Log;

class QuerySecuritySignOut extends AmadeusRequestInSeries
{
    use FileStorageTrait;
    protected int $sequenceNumber;
    protected string $amadeusSessionId;
    protected string $amadeusSecurityToken;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 4;
    }

    public function execute(string $amadeusSessionId, string $amadeusSecurityToken): array
    {
        try {
            $this->amadeusSessionId = $amadeusSessionId;
            $this->amadeusSecurityToken = $amadeusSecurityToken;

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "1.3-security-sign-out-response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "1.3-security-sign-out-response", json_encode($responseArray)
            );

            return $responseArray;

        } catch (\Exception $exception) {
            $e = "Error in QueryFareCheckRulesAction execute:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ['error' => $e];
        }
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader($this->sequenceNumber, $this->amadeusSessionId, $this->amadeusSecurityToken) .
            "
             <soap:Body>
                <Security_SignOut />
             </soap:Body>
           </soap:Envelope>
        ";

        self::storeInFile("xml", "1.3-security-sign-out-request", $xmlRequest);

        return $xmlRequest;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/VLSSOQ_04_1_1A";
    }
}
